<?php

namespace Drupal\bulksms;

/**
 * Bulk SMS Gateway.
 */
class BulkSmsGateway {

  const GATEWAY_URLS = [
    'international' => 'http://bulksms.vsms.net/eapi/submission/send_sms/2/2.0',
    'uk' => 'http://www.bulksms.co.uk/eapi/submission/send_sms/2/2.0',
    'south_africa' => 'http://bulksms.2way.co.za/eapi/submission/send_sms/2/2.0',
    'espana' => 'http://bulksms.com.es/eapi/submission/send_sms/2/2.0',
    'usa' => 'http://usa.bulksms.com/eapi/submission/send_sms/2/2.0',
    'deutschland' => 'http://bulksms.de/eapi/submission/send_sms/2/2.0',
  ];

  const GATEWAYS = [
    'international' => 'BulkSMS International (€)',
    'uk' => 'BulkSMS UK (£)',
    'south_africa' => 'BulkSMS South Africa (R)',
    'espana' => 'BulkSMS España (€)',
    'usa' => 'BulkSMS USA ($)',
    'deutschland' => 'BulkSMS Deutschland (€)',
  ];

  const RESPONSE_STATUS_MESSAGES = [
    0 => 'In progress',
    1 => 'Scheduled',
    22 => 'Internal fatal error',
    23 => 'Authentication failure',
    24 => 'Data validation failed',
    25 => 'You do not have sufficient credits',
    26 => 'Upstream credits not available',
    27 => 'You have exceeded your daily quota',
    28 => 'Upstream quota exceeded',
    40 => 'Temporarily unavailable',
    201 => 'Maximum batch size exceeded',
  ];

  /**
   * An HTTP interface (e.g. curl) to communicate through.
   *
   * @var BulkSmsHttpInterface
   */
  private $http;

  /**
   * The regional gateway to send the SMS through,
   * e.g. 'international' or 'south_africa'.
   *
   * @var string
   */
  private $regionalGateway;

  /**
   * The TCP port to send the request through.
   *
   * @var int
   */
  private $port;

  /**
   * The username used to authenticate with the Bulk SMS service.
   *
   * @var string
   */
  private $username;

  /**
   * The password used to authenticate with the Bulk SMS service.
   *
   * @var string
   */
  private $password;

  /**
   * An optional sender ID.
   *
   * @var string|null
   */
  private $senderId;

  /**
   * A function to call with debugging messages.
   *
   * @var callable|null
   */
  private $debugCallback;

  /**
   * A function to call with the results of each API call.
   *
   * @var callable|null
   */
  private $logCallback;

  /**
   * The test mode, one of 'test_succeed', 'test_fail' or 'live'.
   *
   * @var string
   */
  private $testMode;

  /**
   * Gateway constructor.
   *
   * @param BulkSmsHttpInterface $http
   *   An HTTP interface (e.g. curl) to communicate through.
   * @param string $regionalGateway
   *   The regional gateway to send the SMS through,
   *   e.g. 'international' or 'south_africa'.
   * @param int $port
   *   The TCP port to send the request through.
   * @param string $username
   *   The username used to authenticate with the Bulk SMS service.
   * @param string $password
   *   The password used to authenticate with the Bulk SMS service.
   * @param string|null $senderId
   *   An optional sender ID.
   * @param callable|null $debugCallback
   *   A function to call with debugging messages.
   *   Debugging is disabled if this is NULL.
   * @param callable|null $logCallback
   *   A function to call with the results of each API call.
   *   Logging is disabled if this is NULL.
   * @param string $testMode
   *   The test mode, one of 'test_succeed', 'test_fail' or 'live'.
   */
  public function __construct(
    $http,
    $regionalGateway,
    $port,
    $username,
    $password,
    $senderId = NULL,
    $debugCallback = NULL,
    $logCallback = NULL,
    $testMode = "live"
  ) {
    $this->http = $http;
    $this->regionalGateway = $regionalGateway;
    $this->port = $port;
    $this->username = $username;
    $this->password = $password;
    $this->senderId = $senderId;
    $this->debugCallback = $debugCallback;
    $this->logCallback = $logCallback;
    $this->testMode = $testMode;
  }

  /**
   * Emit a debug message.
   *
   * @param string $message
   *   The message to emit.
   */
  private function debug(string $message) {
    if (is_callable($this->debugCallback)) {
      $func = $this->debugCallback;
      $func($message);
    }
  }

  /**
   * Emit a log record.
   *
   * @param array $numbers
   *   The phone numbers the message was sent to.
   * @param string $message
   *   The message that was sent.
   * @param array $result
   *   The result of the API call.
   */
  private function log(array $numbers, string $message, array $result) {
    if (is_callable($this->logCallback)) {
      $func = $this->logCallback;
      $func($numbers, $message, $result);
    }
  }

  /**
   * Sends a single SMS message to one or more recipients or groups.
   *
   * @param string|array $numbers
   *   One (if string) or more (if array) mobile numbers to send SMS message to.
   *   Numbers should be defined with country code, but without preceding
   *   '+' or '00' (for example 44123123456).
   * @param string $message
   *   Text of the message to be sent.
   * @param array $extra_post_fields
   *   Additional optional parameters to be sent to gateway, as defined in
   *   send_sms docs: https://www.bulksms.co.uk/docs/eapi/submission/send_sms/ .
   *
   * @return array
   *   Result of SMS sending, including following array keys:
   *   - status_code - result code of SMS sending,
   *   - status_message - meaning of result_code,
   *   - status_description - could contain additional useful information
   *     about the result,
   *   - batch_id - optional, depending on the error.
   */
  public function send($numbers, $message, array $extra_post_fields = []): array {
    return $this->callApi('send_sms', $numbers, $message, $extra_post_fields);
  }

  /**
   * Calculate the cost to send an SMS to one or more recipients or groups.
   *
   * @param string|array $numbers
   *   One (if string) or more (if array) mobile numbers to send SMS message to.
   *   Numbers should be defined with country code, but without preceding
   *   '+' or '00' (for example 44123123456).
   * @param string $message
   *   Text of the message to be sent.
   * @param array $extra_post_fields
   *   Additional optional parameters to be sent to gateway, as defined in
   *   send_sms docs: https://www.bulksms.co.uk/docs/eapi/submission/quote_sms/ .
   *
   * @return array
   *   Result of SMS sending, including following array keys:
   *   - status_code - result code of SMS sending,
   *   - status_message - meaning of result_code,
   *   - status_description - could contain additional useful information
   *     about the result,
   *   - quote_total - how many credits it will cost to send given message.
   */
  public function quote($numbers, $message, array $extra_post_fields = []): array {
    return $this->callApi('quote_sms', $numbers, $message, $extra_post_fields);
  }

  /**
   * @return string
   */
  public function getRegionalGateway(): string {
    return $this->regionalGateway;
  }

  /**
   * @return int
   */
  public function getPort(): int {
    return $this->port;
  }

  /**
   * @return string
   */
  public function getUsername(): string {
    return $this->username;
  }

  /**
   * @return string
   */
  public function getPassword(): string {
    return $this->password;
  }

  /**
   * @return string|null
   */
  public function getSenderId(): ?string {
    return $this->senderId;
  }

  /**
   * @return string
   */
  public function getGatewayUrl(): string {
    return self::GATEWAY_URLS[$this->regionalGateway];
  }

  /**
   * Communicate with the BulkSMS service.
   *
   * @param string $service
   *   The type of request to make, e.g. 'quote_sms' or 'send_sms'.
   * @param string|array $numbers
   *   One (if string) or more (if array) mobile numbers to send SMS message to.
   *   Numbers should be defined with country code, but without preceding
   *   '+' or '00' (for example 44123123456).
   * @param string $message
   *   Text of the message to be sent.
   * @param array $extra_post_fields
   *   Additional optional parameters to be sent to gateway, as defined in
   *   send_sms docs: https://www.bulksms.co.uk/docs/eapi/submission/send_sms/ .
   *
   * @return array
   *   An array with status information.
   */
  private function callApi($service, $numbers, $message, array $extra_post_fields = []) {
    // Prepare request params.
    $url = $this->getGatewayUrl();
    if ($service == 'quote_sms') {
      $url = str_replace('/send_sms/', '/quote_sms/', $url);
    }
    // Convert $numbers variable to array if needed.
    if (!is_array($numbers)) {
      $numbers = (array) $numbers;
    }
    // Prepare post fields.
    $post_fields = $this->prepareQueryString($numbers, $message, $extra_post_fields);
    // Send request.
    $result = self::post($url, $this->port, $post_fields);
    // Save log to database.
    $this->log($numbers, $message, $result);
    // Return sending result.
    return $result;
  }

  /**
   * Prepares POST fields for cURL call.
   */
  private function prepareQueryString($numbers, $message, $extra_post_fields = []): string {
    // Prepare basic POST parameters.
    $post_fields = [
      'username' => $this->username,
      'password' => $this->password,
      'message' => $message,
      'msisdn' => implode(',', $numbers),
    ];
    // Add Sender ID if defined.
    if ($this->senderId) {
      $post_fields['sender'] = $this->senderId;
    }
    // Add test mode parameters if required.
    switch ($this->testMode) {
      case 'test_succeed':
        $post_fields['test_always_succeed'] = TRUE;
        break;

      case 'test_fail':
        $post_fields['test_always_fail'] = TRUE;
        break;
    }
    // Add extra POST fields provided in function parameter.
    if ($extra_post_fields) {
      $post_fields += $extra_post_fields;
    }
    // Create query string from POST fields.
    $query_string = http_build_query($post_fields, NULL, '&');

    // Debug.
    $this->debug('POST fields: <pre>' . var_export($post_fields, TRUE) . '</pre>');
    $this->debug('Query string: <pre>' . var_export($query_string, TRUE) . '</pre>');

    return $query_string;
  }

  /**
   * Sends POST request to Bulk SMS gateway.
   */
  private function post($url, $port, $post_fields) {
    $this->http->init($url);
    $this->http->setOption(CURLOPT_PORT, $port);
    $this->http->setOption(CURLOPT_POST, 1);
    $this->http->setOption(CURLOPT_RETURNTRANSFER, 1);
    $this->http->setOption(CURLOPT_POSTFIELDS, $post_fields);
    $this->http->setOption(CURLOPT_TIMEOUT, 20);
    $this->http->setOption(CURLOPT_CONNECTTIMEOUT, 20);
    $http_response = $this->http->execute();
    $http_info = $this->http->getInfo();

    // Debug.
    $this->debug('cURL request info: <pre>' . var_export($http_info, TRUE) . '</pre>');
    $this->debug('Gateway response: <pre>' . var_export($http_response, TRUE) . '</pre>');

    if ($http_response === FALSE) {
      // cURL error.
      $response = [
        'status_code' => $http_response,
        'status_message' => 'cURL error: ' . $this->http->getError(),
      ];
    }
    elseif ($http_info['http_code'] != 200) {
      // Non-200 HTTP status code returned by gateway.
      $response = [
        'status_code' => $http_response,
        'status_message' => 'Error: non-200 HTTP status code returned by cURL call: ' . $http_info['http_code'],
      ];
    }
    else {
      // Correct response received from gateway.
      $response = self::decodeApiResponse($http_response);
    }
    // Close connection.
    $this->http->close();

    // Debug.
    $this->debug('SMS sending result: <pre>' . var_export($response, TRUE) . '</pre>');

    return $response;
  }

  /**
   * Decodes Bulk SMS gateway response.
   */
  private static function decodeApiResponse($response) {
    $response_exploded = explode('|', $response);
    $response = [
      'status_code' => $response_exploded[0],
      'status_message' => self::RESPONSE_STATUS_MESSAGES[$response_exploded[0]],
      'status_description' => $response_exploded[1],
    ];
    // 3rd and last field returned by the gateway will be different
    // depending whether "send_sms" or "quote_sms" API method was called.
    // For "send_sms" API method its name should be "batch_id",
    // while for "quote_sms" it should be "quote_total".
    $last_field_name = ($response_exploded[1] == 'Quotation issued') ? 'quote_total' : 'batch_id';
    $response[$last_field_name] = trim($response_exploded[2]);
    return $response;
  }

}
