<?php

use Drupal\bulksms\BulkSmsGateway;
use Drupal\bulksms\BulkSmsCurl;
use PHPUnit\Framework\TestCase;

/**
 * Test the BulkSmsGateway class.
 */
class BulkSmsGatewayTest extends TestCase {

  /**
   * Test constructing the gateway and sending a message in live mode.
   *
   * @throws \PHPUnit\Framework\MockObject\RuntimeException
   */
  public function testLiveSend() {
    $fake_curl = $this->getMockBuilder(BulkSmsCurl::class)->getMock();
    $fake_curl
      ->method('execute')
      ->willReturn("0|IN_PROGRESS|1015616245\n");
    $fake_curl
      ->method('getInfo')
      ->willReturn([
        'url' => 'http://bulksms.2way.co.za/eapi/submission/send_sms/2/2.0',
        'content_type' => 'text/plain; charset=iso-8859-1',
        'http_code' => 200,
        'header_size' => 259,
        'request_size' => 250,
        'filetime' => -1,
        'ssl_verify_result' => 0,
        'redirect_count' => 0,
        'total_time' => 3.617394,
        'namelookup_time' => 1.510678,
        'connect_time' => 2.364298,
        'pretransfer_time' => 2.364479,
        'size_upload' => 88.0,
        'size_download' => 25.0,
        'speed_download' => 6.0,
        'speed_upload' => 24.0,
        'download_content_length' => 25.0,
        'upload_content_length' => 88.0,
        'starttransfer_time' => 3.61734,
        'redirect_time' => 0.0,
        'redirect_url' => '',
        'primary_ip' => '197.157.88.213',
        'certinfo' => [],
        'primary_port' => 5567,
        'local_ip' => '192.168.43.208',
        'local_port' => 38688,
      ]);

    $log = [];
    $logger = function ($number, $message, $result) use (&$log) {
      $log[] = [
        'number' => $number,
        'message' => $message,
        'result' => $result,
      ];
    };

    $debug_log = [];
    $debug_logger = function ($message) use (&$debug_log) {
      $debug_log[] = $message;
    };

    $gateway = new BulkSmsGateway(
      $fake_curl,
      'south_africa',
      1234,
      'dolfie',
      'wagwoord',
      'rsc',
      $debug_logger,
      $logger,
      'live'
    );

    // Test the getters.
    $this->assertEquals("south_africa", $gateway->getRegionalGateway());
    $this->assertEquals("http://bulksms.2way.co.za/eapi/submission/send_sms/2/2.0", $gateway->getGatewayUrl());
    $this->assertEquals(1234, $gateway->getPort());
    $this->assertEquals("dolfie", $gateway->getUsername());
    $this->assertEquals("wagwoord", $gateway->getPassword());
    $this->assertEquals("rsc", $gateway->getSenderId());

    // Test sending an SMS.
    $this->assertEquals(
      [
        'status_code' => '0',
        'status_message' => 'In progress',
        'status_description' => 'IN_PROGRESS',
        'batch_id' => '1015616245',
      ],
      $gateway->send("+27760297198", "Test message from PHPUnit")
    );

    // Check the logs.
    $this->assertEquals(
      [
        0 => 'POST fields: <pre>array (
  \'username\' => \'dolfie\',
  \'password\' => \'wagwoord\',
  \'message\' => \'Test message from PHPUnit\',
  \'msisdn\' => \'+27760297198\',
  \'sender\' => \'rsc\',
)</pre>',
        1 => 'Query string: <pre>\'username=dolfie&password=wagwoord&message=Test+message+from+PHPUnit&msisdn=%2B27760297198&sender=rsc\'</pre>',
        2 => 'cURL request info: <pre>array (
  \'url\' => \'http://bulksms.2way.co.za/eapi/submission/send_sms/2/2.0\',
  \'content_type\' => \'text/plain; charset=iso-8859-1\',
  \'http_code\' => 200,
  \'header_size\' => 259,
  \'request_size\' => 250,
  \'filetime\' => -1,
  \'ssl_verify_result\' => 0,
  \'redirect_count\' => 0,
  \'total_time\' => 3.617394,
  \'namelookup_time\' => 1.510678,
  \'connect_time\' => 2.364298,
  \'pretransfer_time\' => 2.364479,
  \'size_upload\' => 88.0,
  \'size_download\' => 25.0,
  \'speed_download\' => 6.0,
  \'speed_upload\' => 24.0,
  \'download_content_length\' => 25.0,
  \'upload_content_length\' => 88.0,
  \'starttransfer_time\' => 3.61734,
  \'redirect_time\' => 0.0,
  \'redirect_url\' => \'\',
  \'primary_ip\' => \'197.157.88.213\',
  \'certinfo\' => 
  array (
  ),
  \'primary_port\' => 5567,
  \'local_ip\' => \'192.168.43.208\',
  \'local_port\' => 38688,
)</pre>',
        3 => 'Gateway response: <pre>\'0|IN_PROGRESS|1015616245
\'</pre>',
        4 => 'SMS sending result: <pre>array (
  \'status_code\' => \'0\',
  \'status_message\' => \'In progress\',
  \'status_description\' => \'IN_PROGRESS\',
  \'batch_id\' => \'1015616245\',
)</pre>',
      ],
      $debug_log
    );

    $this->assertEquals(
      [
        0 =>
          [
            'number' =>
              [
                0 => '+27760297198',
              ],
            'message' => 'Test message from PHPUnit',
            'result' =>
              [
                'status_code' => '0',
                'status_message' => 'In progress',
                'status_description' => 'IN_PROGRESS',
                'batch_id' => '1015616245',
              ],
          ],
      ],
      $log
    );
  }

  /**
   * Test getting a quote in live mode.
   *
   * @throws \PHPUnit\Framework\MockObject\RuntimeException
   */
  public function testLiveQuote() {
    $fake_curl = $this->getMockBuilder(BulkSmsCurl::class)->getMock();
    $fake_curl
      ->method('execute')
      ->willReturn("0|Quotation issued|1.0\n");
    $fake_curl
      ->method('getInfo')
      ->willReturn([
        'url' => 'http://bulksms.2way.co.za/eapi/submission/quote_sms/2/2.0',
        'content_type' => 'text/plain; charset=iso-8859-1',
        'http_code' => 200,
        'header_size' => 259,
        'request_size' => 251,
        'filetime' => -1,
        'ssl_verify_result' => 0,
        'redirect_count' => 0,
        'total_time' => 0.43532400000000004,
        'namelookup_time' => 0.25331099999999995,
        'connect_time' => 0.34327399999999997,
        'pretransfer_time' => 0.34342399999999995,
        'size_upload' => 88.0,
        'size_download' => 23.0,
        'speed_download' => 52.0,
        'speed_upload' => 202.0,
        'download_content_length' => 23.0,
        'upload_content_length' => 88.0,
        'starttransfer_time' => 0.43527000000000005,
        'redirect_time' => 0.0,
        'redirect_url' => '',
        'primary_ip' => '197.157.88.213',
        'certinfo' => [],
        'primary_port' => 5567,
        'local_ip' => '192.168.43.208',
        'local_port' => 38834,
      ]);

    $log = [];
    $logger = function ($number, $message, $result) use (&$log) {
      $log[] = [
        'number' => $number,
        'message' => $message,
        'result' => $result,
      ];
    };

    $debug_log = [];
    $debug_logger = function ($message) use (&$debug_log) {
      $debug_log[] = $message;
    };

    $gateway = new BulkSmsGateway(
      $fake_curl,
      'international',
      2345,
      'dolf',
      'wag',
      'RSC',
      $debug_logger,
      $logger,
      'live'
    );

    // Test getting a quote
    $this->assertEquals(
      [
        'status_code' => '0',
        'status_message' => 'In progress',
        'status_description' => 'Quotation issued',
        'quote_total' => '1.0',
      ],
      $gateway->quote("+27760297198", "Hoeveel gaan dit kos?")
    );

    // Check the logs.
    $this->assertEquals(
      [
        0 => 'POST fields: <pre>array (
  \'username\' => \'dolf\',
  \'password\' => \'wag\',
  \'message\' => \'Hoeveel gaan dit kos?\',
  \'msisdn\' => \'+27760297198\',
  \'sender\' => \'RSC\',
)</pre>',
        1 => 'Query string: <pre>\'username=dolf&password=wag&message=Hoeveel+gaan+dit+kos%3F&msisdn=%2B27760297198&sender=RSC\'</pre>',
        2 => 'cURL request info: <pre>array (
  \'url\' => \'http://bulksms.2way.co.za/eapi/submission/quote_sms/2/2.0\',
  \'content_type\' => \'text/plain; charset=iso-8859-1\',
  \'http_code\' => 200,
  \'header_size\' => 259,
  \'request_size\' => 251,
  \'filetime\' => -1,
  \'ssl_verify_result\' => 0,
  \'redirect_count\' => 0,
  \'total_time\' => 0.43532400000000004,
  \'namelookup_time\' => 0.25331099999999995,
  \'connect_time\' => 0.34327399999999997,
  \'pretransfer_time\' => 0.34342399999999995,
  \'size_upload\' => 88.0,
  \'size_download\' => 23.0,
  \'speed_download\' => 52.0,
  \'speed_upload\' => 202.0,
  \'download_content_length\' => 23.0,
  \'upload_content_length\' => 88.0,
  \'starttransfer_time\' => 0.43527000000000005,
  \'redirect_time\' => 0.0,
  \'redirect_url\' => \'\',
  \'primary_ip\' => \'197.157.88.213\',
  \'certinfo\' => 
  array (
  ),
  \'primary_port\' => 5567,
  \'local_ip\' => \'192.168.43.208\',
  \'local_port\' => 38834,
)</pre>',
        3 => 'Gateway response: <pre>\'0|Quotation issued|1.0
\'</pre>',
        4 => 'SMS sending result: <pre>array (
  \'status_code\' => \'0\',
  \'status_message\' => \'In progress\',
  \'status_description\' => \'Quotation issued\',
  \'quote_total\' => \'1.0\',
)</pre>',
      ],
      $debug_log
    );

    $this->assertEquals(
      [
        0 =>
          [
            'number' =>
              [
                0 => '+27760297198',
              ],
            'message' => 'Hoeveel gaan dit kos?',
            'result' =>
              [
                'status_code' => '0',
                'status_message' => 'In progress',
                'status_description' => 'Quotation issued',
                'quote_total' => '1.0',
              ],
          ],
      ],
      $log
    );
  }

  /**
   * Test what happens when we don't get a timely HTTP response.
   */
  public function testTimeout() {
    $fake_curl = $this->getMockBuilder(BulkSmsCurl::class)->getMock();
    $fake_curl
      ->method('execute')
      ->willReturn(FALSE);
    $fake_curl
      ->method('getInfo')
      ->willReturn([
        'url' => 'http://bulksms.2way.co.za/eapi/submission/send_sms/2/2.0',
        'content_type' => NULL,
        'http_code' => 0,
        'header_size' => 0,
        'request_size' => 0,
        'filetime' => -1,
        'ssl_verify_result' => 0,
        'redirect_count' => 0,
        'total_time' => 20.003565,
        'namelookup_time' => 2.511879,
        'connect_time' => 0.0,
        'pretransfer_time' => 0.0,
        'size_upload' => 0.0,
        'size_download' => 0.0,
        'speed_download' => 0.0,
        'speed_upload' => 0.0,
        'download_content_length' => -1.0,
        'upload_content_length' => -1.0,
        'starttransfer_time' => 0.0,
        'redirect_time' => 0.0,
        'redirect_url' => '',
        'primary_ip' => '',
        'certinfo' => [],
        'primary_port' => 0,
        'local_ip' => '',
        'local_port' => 0,
      ]);
    $fake_curl
      ->method('getError')
      ->willReturn('Connection timed out after 20000 milliseconds');

    $gateway = new BulkSmsGateway(
      $fake_curl,
      'international',
      2345,
      'dolf',
      'wag',
      'RSC',
      NULL,
      NULL,
      'live'
    );

    $this->assertEquals(
      [
        'status_code' => FALSE,
        'status_message' => 'cURL error: Connection timed out after 20000 milliseconds',
      ],
      $gateway->quote("+27760297198", "Hoeveel gaan dit kos?")
    );
  }

  /**
   * Test what happens when we have invalid credentials.
   */
  public function testInvalidCredentials() {
    $fake_curl = $this->getMockBuilder(BulkSmsCurl::class)->getMock();
    $fake_curl
      ->method('execute')
      ->willReturn("23|invalid credentials (username was: dolfie)|\n");
    $fake_curl
      ->method('getInfo')
      ->willReturn([
        'url' => 'http://bulksms.2way.co.za/eapi/submission/quote_sms/2/2.0',
        'content_type' => 'text/plain; charset=iso-8859-1',
        'http_code' => 200,
        'header_size' => 259,
        'request_size' => 235,
        'filetime' => -1,
        'ssl_verify_result' => 0,
        'redirect_count' => 0,
        'total_time' => 0.248004,
        'namelookup_time' => 0.0612,
        'connect_time' => 0.147797,
        'pretransfer_time' => 0.14792,
        'size_upload' => 72.0,
        'size_download' => 56.0,
        'speed_download' => 225.0,
        'speed_upload' => 290.0,
        'download_content_length' => 56.0,
        'upload_content_length' => 72.0,
        'starttransfer_time' => 0.247953,
        'redirect_time' => 0.0,
        'redirect_url' => '',
        'primary_ip' => '197.157.88.213',
        'certinfo' => [],
        'primary_port' => 5567,
        'local_ip' => '192.168.43.208',
        'local_port' => 39382,
      ]);

    $gateway = new BulkSmsGateway(
      $fake_curl,
      'international',
      2345,
      'dolfie',
      'wag',
      NULL,
      NULL,
      NULL,
      'live'
    );

    $this->assertEquals(
      [
        'status_code' => '23',
        'status_message' => 'Authentication failure',
        'status_description' => 'invalid credentials (username was: dolfie)',
        'batch_id' => '',
      ],
      $gateway->quote("+27760297198", "Mag ek inkom?")
    );
  }
}
